var clearFieldsUrl = "{% url 'core_curate_clear_fields' %}";
var saveFormUrl = "{% url 'core_curate_save_form' %}";
var validateFormUrl = "{% url 'core_curate_validate_form' %}";
var cancelChangesUrl = "{% url 'core_curate_cancel_changes' %}";
var cancelFormUrl = "{% url 'core_curate_cancel_form' %}";
var curateIndexUrl = "{% url 'core_curate_index' %}";
var getTemplateUrl = "{% url 'core_main_app_rest_template_detail'  'template_id'%}";
var proceedToReviewUrl = "{% url 'core_curate_view_data' 'curate_data_structure_id' %}"
var openXMLFormUrl = "{% url 'core_curate_app_xml_text_editor_view' %}";
var openJSONFormUrl = "{% url 'core_curate_app_json_text_editor_view' %}";
var downloadDocumentUrl = "{% url 'core_curate_download_document' data.data_structure.id %}";
var downloadTemplateUrl = "{% url 'core_curate_download_template' data.data_structure.id %}";
