.tooltip, .popover {
    z-index: 1 !important;
}

.center-text{
	text-align: center;
}

.warning-modal-container {
    display: none;
    opacity: 0;
}

#valid-modal-toggle-warning {
    width: 100%;
    color: #004ad6;
    cursor: pointer;
}

.warning-icon {
    color: #ff8f00;
}

.alert.alert-warning {
    margin: 20px;
}

.warning-popover {
    max-width: 395px !important;

    border: none;
}

.warning-popover-text {
    background-color: #ea8d00;

    border-bottom: none;
    border-radius: 5px;

    padding: 0.2rem;

    color: white;
    font-size: 14px;
    text-align: center;
}

.warning-popover-arrow::after {
    border-right-color: #ea8d00 !important;
    border: none;
}
